/*
 * Automat.cpp
 *
 * Template:
 *  Created on: Jul 5, 2012
 *      Author: knad0001
 * Class and overall file:
 *  Created on: Apr 16, 2014
 *      Author: Skia
 */

#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <cctype>
#include "Automat.h"


/*
 * **************************
 * Constructor and destructor
 * **************************
 */

const int Automat::m_matrix[NR_OF_CHAR_TYPE][NR_OF_STATE] = {
              /* START      |IDENTIFIER  |INTEGER  |SIGN |EQ       |D_EQ |T_EQ |EQ_DIFF     |EQ_DIFF_EQ|SLASH  |COM  |COM_M|END|ERROR */
/* SIGN */     {S_SIGN      ,S_END       ,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END},
/* EQ */       {S_EQ        ,S_END       ,S_END    ,S_END,S_D_EQ   ,S_END,S_END,S_EQ_DIFF_EQ,S_END     ,S_END  ,S_END,S_END},
/* DIFF */     {S_SIGN      ,S_END       ,S_END    ,S_END,S_EQ_DIFF,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END},
/* SLASH */    {S_SLASH     ,S_END       ,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_COM  ,S_END,S_END},
/* STAR */     {S_SIGN      ,S_END       ,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_COM_M,S_END,S_END},
/* DIGIT */    {S_INTEGER   ,S_IDENTIFIER,S_INTEGER,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END},
/* LETTER */   {S_IDENTIFIER,S_IDENTIFIER,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END},
/* SPACE */    {S_START     ,S_END       ,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END},
/* UNKNOWN */  {S_ERROR     ,S_END       ,S_END    ,S_END,S_END    ,S_END,S_END,S_END       ,S_END     ,S_END  ,S_END,S_END}
};

Automat::Automat() : m_type(T_UNKNOWN), m_state(S_START), m_back(0) {
}

Automat::~Automat() {
}


/* Look at the current state and determine the token type */
void Automat::setTokenType( void )
{
    switch(m_state) {
        case S_IDENTIFIER:
            m_type = T_IDENTIFIER;
            break;
        case S_INTEGER:
            m_type = T_INTEGER;
            break;
        case S_SIGN:
            m_type = getSignType();
            break;
        case S_EQ:
            m_type = T_ASSIGN;
            break;
        case S_D_EQ:
            m_type = T_EQUAL;
            break;
        case S_EQ_DIFF_EQ:
            m_type = T_UNEQUAL;
            break;
        case S_EQ_DIFF:
            m_type = T_ASSIGN;
            m_back++;
            break;
        case S_SLASH:
            m_type = T_DIVIDE;
            break;
        case S_COM:
            m_type = T_COM;
            break;
        case S_COM_M:
            m_type = T_COM_M;
            break;
        default:
            m_type = T_UNKNOWN;
            break;
    }
}

/* Return the type of the passed char */
TokenType Automat::getSignType( void ) const
{
    switch(m_currentChar) {
        case '+':
            return T_PLUS;
            break;
        case '-':
            return T_MINUS;
            break;
        case '*':
            return T_MULTI;
            break;
        case '/':
            return T_DIVIDE;
            break;
        case '<':
            return T_LESSTHAN;
            break;
        case '>':
            return T_MORETHAN;
            break;
        case '!':
            return T_EXCLAMATION;
            break;
        case '&':
            return T_AND;
            break;
        case ';':
            return T_SEMICOLON;
            break;
        case '(':
            return T_LPARENT;
            break;
        case ')':
            return T_RPARENT;
            break;
        case '[':
            return T_LBRACKET;
            break;
        case ']':
            return T_RBRACKET;
            break;
        case '{':
            return T_LBRACE;
            break;
        case '}':
            return T_RBRACE;
            break;
        default:
            return T_UNKNOWN;
            break;
    }
}

/* Returns the type of the current read char */
CharType Automat::typeOfChar( char c ) const
{
    if(isdigit(c))
        return C_DIGIT;
    else if(isalpha(c))
        return C_LETTER;
    else if(isspace(c) || c == '\0')
        return C_SPACE;
    else if(c == '=')
        return C_EQ;
    else if(c == '!')
        return C_DIFF;
    else if(c == '/')
        return C_SLASH;
    else if(c == '*')
        return C_STAR;
    else if(c == '+' ||
            c == '-' ||
            c == '<' ||
            c == '>' ||
            c == '&' ||
            c == ';' ||
            c == '(' ||
            c == ')' ||
            c == '{' ||
            c == '}' ||
            c == '[' ||
            c == ']')
        return C_SIGN;
    else
        return C_UNKNOWN;
}


/*
 * **************************
 * Getters (public functions)
 * **************************
 */

State Automat::getState( void ) const
{
    return m_state;
}

TokenType Automat::getTokenType( void ) const
{
    return m_type;
}

int Automat::getBack( void ) const
{
    return m_back;
}


/* The main function that read a char and return true if a token has been
 * recognized, and false otherwise */
bool Automat::readChar( char c )
{
    State tmpState;

    if(m_state == S_END) { // If last state was END, then we start a new token
        m_state = S_START;
        m_back = 0;
    }

    tmpState = (State)m_matrix[typeOfChar(c)][m_state]; // Set the tmp state

    if(tmpState == S_ERROR) {
        setTokenType();
        m_state = S_END;
        return true;
    }

    if(tmpState == S_SIGN) // No need to get next char to identify the sign
        m_state = S_SIGN;

    if(tmpState == S_END && !isspace(c)) // If we come to the end of the token, we'll need to rescan the current char in the next token
        m_back++;

    if(m_state == S_SIGN || tmpState == S_END) { // Set a few attributes before returning that a token has been recognized
        m_currentChar = c;
        setTokenType();
        m_state = S_END;

        return true;
    } else { // Token not yet recognized
        m_state = tmpState;

        return false;
    }

}


