/*
 * Scanner.cpp
 *
 *  Created on: Sep 26, 2012
 *      Author: knad0001
 *  Class and overall file:
 *  Created on: Apr 19, 2014
 *      Author: mino
 */

#include <Scanner.h>
#include <Token.h>

/*
===========================================================================

SCANNER

===========================================================================
*/

/*
 * **************************
 * Constructor and destructor
 * **************************
 */

Scanner::Scanner( char *filepath, Symboltable *symtab )
{
    buffer = new Buffer( filepath );
    automat = new Automat();
    this->symtab = symtab;
}

Scanner::~Scanner()
{
    delete buffer;
    delete automat;
}

/*
 * **************************
 * Private functions
 * **************************
 */

/* Creates a new Token object if all conditions are met.
 * It also interacts with the symboltable in case of identifier */
Token *Scanner::createToken( TokenType ttype, int row, int column, char *lexem )
{
    int         value = 0;
    long int    li;
    symtabkey_t key;
    Token       *t = NULL;
    TokenType type = ttype;

    switch( ttype ) {
        case T_INTEGER:
            errno = 0;
            li = strtol( lexem, NULL, 10 );
            if( errno == ERANGE ) {
                std::cout << "Integer conversion overflow\n";
                exit(1); // TODO
            }
            if ( li < INT_MIN || li > INT_MAX ) {
                std::cout << "Integer overflow\n";
                exit(1); // TODO
            }
            value = static_cast<int>(li);
            break;
        case T_IDENTIFIER:
            key = symtab->insert( lexem );
            type = key->getInfo()->getType();
            break;
        case T_UNKNOWN:
            value = static_cast<int>(lexem[0]);
        default:
            break;
    }

    t = new Token( type, row, column, key, value );

    return t;
}

/* Skips all chars of what is present in the buffer until
 * it encounters a closing condition. */
// FIXME this function exits out if a comment reaches EOF
void Scanner::skipComment( bool multiline )
{
    char ch1, ch2;
    if( multiline ) {
        while(1) {
            ch1 = buffer->getChar();
            if( ch1 == '*' ) { // we expect a closing "*/" or EOF to exit out
                ch2 = buffer->getChar();
                if( ch2 == '/' ) {
                    break;
                } else {
                    buffer->unGetChar( 1 );
                }
            } else if( ch1 == EOF_ENCODING ) {
                printf( "Reached EOF on multiline comment (FIXME)\n" );
                exit(1);
            }
        }
    } else {
        while(1) {
            ch1 = buffer->getChar();
            if( ch1 == '\n' ) { // we expect a closing "\n" or EOF to exit out
                break;
            } else if( ch1 == EOF_ENCODING ) {
                printf( "Reached EOF on singleline comment (FIXME)\n" );
                exit(1);
            }
        }
    }
}

/*
 * **************************
 * Public functions
 * **************************
 */

/* The main interface to parser. It works in conjunction with all modules
 * to construct a token. */
Token *Scanner::nextToken( void )
{
    //char        ch, ch_temp;
    char ch;
    Token       *t = NULL;

    while( 42 ) {

        // get next char and do stuff to it
        ch = buffer->getChar();

        // we reached the end of the file, exit out
        if( ch == EOF_ENCODING ) {
            // if we are already in, we need to construct a token
            if( automat->getState() != S_START ) {
                automat->readChar( ' ' ); // reset the automat TODO this needs to be fixed
                break;
            } else {
                return NULL; // the end of scanning
            }
        }

        // filter out whitespace in some cases | Why doing that here? Quite the
        // same is done in the automaton...
        // @Skia: it is done to fix lexem in buffer ... at end of testFile write: "abc;     def"
        // Test this! Watch output of def ... I have not looked at your automat, but as you can
        // see lexem def gets messed up with whitespace

        /*
        if( isspace( ch ) ) {
            // we are at the beginning of a token cycle, ignore whitespace
            // so chars don't end up in Buffer::lexem[]
            if( automat->getState() == S_START || automat->getState() == S_END ) {
                automat->readChar( ' ' ); // reset the automat TODO this needs to be fixed
                // the whole code is messy, even the if condition
                buffer->setNewLexem();
                continue;
            }
        }
        */


        // nothing of the above happened, so we give the char to automat
        if( automat->readChar( ch ) ) { // the automat has recognized something
            if (automat->getTokenType() == T_COM) { // it's a single line comment
                skipComment(false);
                buffer->setNewLexem();
                continue;
            } else if (automat->getTokenType() == T_COM_M) { // multi line comment
                skipComment(true);
                buffer->setNewLexem();
                continue;
            } else {
                break;
            }
        }
    }

    // we have a token to construct
    buffer->unGetChar( automat->getBack() );
    t = createToken( automat->getTokenType(), buffer->getRow(), buffer->getColumn(), buffer->getLexem() );
    buffer->setNewLexem();
    return t;
}

