#include <Scanner.h>
#include <Token.h>
#include <iostream>
#include <fstream>

using namespace std;

#define OUTPUTFILE_NAME "out.txt"

void printTokenType( TokenType t, ofstream *outputFile )
{
    switch(t) {
        case T_PLUS:
            *outputFile << "Token Plus         ";
            break;
        case T_MINUS:
            *outputFile << "Token Minus        ";
            break;
        case T_DIVIDE:
            *outputFile << "Token Divide       ";
            break;
        case T_MULTI:
            *outputFile << "Token Multi        ";
            break;
        case T_LESSTHAN:
            *outputFile << "Token LessThan     ";
            break;
        case T_MORETHAN:
            *outputFile << "Token MoreThan     ";
            break;
        case T_ASSIGN:
            *outputFile << "Token Assign       ";
            break;
        case T_EQUAL:
            *outputFile << "Token Equal        ";
            break;
        case T_UNEQUAL:
            *outputFile << "Token Unequal      ";
            break;
        case T_EXCLAMATION:
            *outputFile << "Token Exclamation  ";
            break;
        case T_AND:
            *outputFile << "Token And          ";
            break;
        case T_SEMICOLON:
            *outputFile << "Token Semicolon    ";
            break;
        case T_LPARENT:
            *outputFile << "Token LPar         ";
            break;
        case T_RPARENT:
            *outputFile << "Token RPar         ";
            break;
        case T_LBRACKET:
            *outputFile << "Token LBrack       ";
            break;
        case T_RBRACKET:
            *outputFile << "Token RBrack       ";
            break;
        case T_LBRACE:
            *outputFile << "Token LBrace       ";
            break;
        case T_RBRACE:
            *outputFile << "Token RBrace       ";
            break;
        case T_INTEGER:
            *outputFile << "Token Integer      ";
            break;
        case T_IF:
            *outputFile << "Token If           ";
            break;
        case T_WHILE:
            *outputFile << "Token While        ";
            break;
        case T_INT:
            *outputFile << "Token Int          ";
            break;
        case T_IDENTIFIER:
            *outputFile << "Token Identifier   ";
            break;
        case T_END:
            *outputFile << "Token End          ";
            break;
        default:
            *outputFile << "Token Unknown      ";
            break;
    }
}

void printToken( Token *t, ofstream *outputFile )
{
    char *lexem;
    SymtabEntry *ste;
    Information *info;

    // an error token is printed to error
    if( t->getTokenType() == T_UNKNOWN ) {
        std::cerr << "unknown Token Line: " << t->getRow() <<
        " Column: " << t->getColumn() <<
        "\tSymbol: " << static_cast<char>(t->getValue()) << std::endl;;
        return;
    }

    printTokenType( t->getTokenType(), outputFile );
    *outputFile << "Line: " << t->getRow() << " Column: " << t->getColumn();
    if( t->getTokenType() == T_IDENTIFIER ) {
        ste = static_cast<SymtabEntry *>( t->getKey() );
        info = ste->getInfo();
        lexem = info->getName();
        *outputFile << "\tLexem: " << lexem;
    }

    if( t->getTokenType() == T_INTEGER ) {
        *outputFile << "\tValue: " << t->getValue();
    }

    *outputFile << std::endl;
}

int main(int argc, char **argv)
{
    ofstream outputFile;

    if ( argc < 2 ) {
        std::cout << "Error: missing arguments" << std::endl;
        return EXIT_FAILURE;
    }

    outputFile.open( OUTPUTFILE_NAME );

    Token           *t = NULL;
    Symboltable     *symtab = new Symboltable();
    Scanner         *scanner = new Scanner( argv[1], symtab );

    while( (t = scanner->nextToken()) ) {
        printToken(t, &outputFile);
        delete t;
    }

    //symtab->print();

    delete scanner;
    delete symtab;
    outputFile.close();

    return EXIT_SUCCESS;
}

