#include <Token.h>
#include <iostream>


/*
===========================================================================

TOKEN

===========================================================================
*/

/*
 * **************************
 * Constructor and destructor
 * **************************
 */

Token::Token( TokenType ttype, int row, int column, symtabkey_t key, int value )
{
    this->ttype = ttype;
    this->row = row;
    this->column = column;

    if( ttype == T_IDENTIFIER ) {
        this->key = key;
    } else {
        this->key = NULL; // special keycode showing invalid key
        // seems obvious at first, but a key can also be something
        // different, for example an integer, and if we change the
        // key, we must accommodate for it it here. <- way
        // too long comment btw ...
    }

    if( ttype == T_INTEGER || ttype == T_UNKNOWN ) {
        this->value = value;
    } else {
        this->value = 0; // not needed, but my tourette forced me ;)
    }
}

Token::~Token()
{
    // TODO we do not support deletion of symtabentries
    // so the token just dies, and the symtabentry dies
    // on its own when the symboltable dies ...
}

/*
 * **************************
 * Getters (public functions)
 * **************************
 */

TokenType Token::getTokenType( void ) const
{
    return ttype;
}

int Token::getRow( void ) const
{
    return row;
}

int Token::getColumn( void ) const
{
    return column;
}

symtabkey_t Token::getKey( void ) const
{
    return key;
}

int Token::getValue( void ) const
{
    return value;
}

void Token::setTokenType( TokenType ttype )
{
    this->ttype = ttype;
}

void Token::setRow( int row )
{
    this->row = row;
}

void Token::setColumn( int column )
{
    this->column = column;
}

void Token::setKey( symtabkey_t key )
{
    this->key = key;
}

void Token::setValue( int value )
{
    this->value = value;
}

void Token::pType(std::ostream &s) const
{
    switch(this->ttype) {
        case T_PLUS:
            s << "Plus";
            break;
        case T_MINUS:
            s << "Minus";
            break;
        case T_DIVIDE:
            s << "Divide";
            break;
        case T_MULTI:
            s << "Multi";
            break;
        case T_LESSTHAN:
            s << "LessThan";
            break;
        case T_MORETHAN:
            s << "MoreThan";
            break;
        case T_ASSIGN:
            s << "Assign";
            break;
        case T_EQUAL:
            s << "Equal";
            break;
        case T_UNEQUAL:
            s << "Unequal";
            break;
        case T_EXCLAMATION:
            s << "Exclamation";
            break;
        case T_AND:
            s << "And";
            break;
        case T_SEMICOLON:
            s << "Semicolon";
            break;
        case T_LPARENT:
            s << "LPar";
            break;
        case T_RPARENT:
            s << "RPar";
            break;
        case T_LBRACKET:
            s << "LBrack";
            break;
        case T_RBRACKET:
            s << "RBrack";
            break;
        case T_LBRACE:
            s << "LBrace";
            break;
        case T_RBRACE:
            s << "RBrace";
            break;
        case T_INTEGER:
            s << "Integer";
            break;
        case T_IF:
            s << "If";
            break;
        case T_WHILE:
            s << "While";
            break;
        case T_INT:
            s << "Int";
            break;
        case T_IDENTIFIER:
            s << "Identifier";
            break;
        case T_END:
            s << "End";
            break;
        default:
            s << "Unknown";
            break;
    }
}

void Token::pInfo(std::ostream &s) const
{
    this->pType(s);
    s << " (Row " << this->row << ", Col " << this->column << ")" << std::endl;
}

std::ostream &operator<<( std::ostream &s, Token const* t)
{
    t->pType(s);
    return s;
}


void printTokenType( TokenType t )
{
    //std::cout << t << std::endl;
    switch(t) {
        case T_PLUS:
            std::cout << "Token Plus         \n";
            break;
        case T_MINUS:
            std::cout << "Token Minus        \n";
            break;
        case T_DIVIDE:
            std::cout << "Token Divide       \n";
            break;
        case T_MULTI:
            std::cout << "Token Multi        \n";
            break;
        case T_LESSTHAN:
            std::cout << "Token LessThan     \n";
            break;
        case T_MORETHAN:
            std::cout << "Token MoreThan     \n";
            break;
        case T_ASSIGN:
            std::cout << "Token Assign       \n";
            break;
        case T_EQUAL:
            std::cout << "Token Equal        \n";
            break;
        case T_UNEQUAL:
            std::cout << "Token Unequal      \n";
            break;
        case T_EXCLAMATION:
            std::cout << "Token Exclamation  \n";
            break;
        case T_AND:
            std::cout << "Token And          \n";
            break;
        case T_SEMICOLON:
            std::cout << "Token Semicolon    \n";
            break;
        case T_LPARENT:
            std::cout << "Token LPar         \n";
            break;
        case T_RPARENT:
            std::cout << "Token RPar         \n";
            break;
        case T_LBRACKET:
            std::cout << "Token LBrack       \n";
            break;
        case T_RBRACKET:
            std::cout << "Token RBrack       \n";
            break;
        case T_LBRACE:
            std::cout << "Token LBrace       \n";
            break;
        case T_RBRACE:
            std::cout << "Token RBrace       \n";
            break;
        case T_INTEGER:
            std::cout << "Token Integer      \n";
            break;
        case T_IF:
            std::cout << "Token If           \n";
            break;
        case T_WHILE:
            std::cout << "Token While        \n";
            break;
        case T_INT:
            std::cout << "Token Int          \n";
            break;
        case T_IDENTIFIER:
            std::cout << "Token Identifier   \n";
            break;
        case T_END:
            std::cout << "Token End          \n";
            break;
        default:
            std::cout << "Token Unknown      \n";
            break;
    }
}

