/*
 * Symboltable.cpp
 *
 *  Created on: Sep 26, 2012
 *      Author: knad0001
 *  Class and overall file:
 *  Created on: Apr 27, 2014
 *      Author: mino & Skia
 */

#include <Symboltable.h>
#include <Token.h>

/*
===========================================================================

MISC (MOVE ME LATER, MAYBE INTO A SELF-ROLLED LIB ACCESSIBLE TO ALL MODULES)

===========================================================================
*/

// safe strcpy that ensures a trailing zero
// function taken from:
// https://github.com/id-Software/Quake-III-Arena/blob/master/code/game/q_shared.c#L711
void Safe_strncpyz( char *dest, const char *src, int destsize )
{
    if ( !dest ) {
        std::cout << "Safe_strncpyz: NULL dest\n";
        exit(1);
    }

    if ( !src ) {
        std::cout << "Safe_strncpyz: NULL src\n";
        exit(1);
    }

    if ( destsize < 1 ) {
        std::cout << "Safe_strncpyz: destsize < 1\n";
        exit(1);
    }

    strncpy( dest, src, destsize-1 );
    dest[destsize-1] = '\0';
}

/*
===========================================================================

STRINGTABLE

===========================================================================
*/

StringTab::StringTab()
{
	char	*p = NULL;

	posix_memalign( (void **)&p, PAGE_SIZE, STRINGTAB_BUFSIZE );
	if ( p != NULL ) {
        this->space = p;
		this->freeP = p;
		this->freeSpace = STRINGTAB_BUFSIZE;
	} else {
		this->freeP = NULL;
		this->freeSpace = 0;
	}
}

StringTab::~StringTab()
{
    free(this->space);
}

char *StringTab::insert( char *lexem, int size )
{
	char *tmp = this->freeP;

	if ( size < this->freeSpace ) {
		memcpy( this->freeP, lexem, size + 1 );
		this->freeP[size] = '\0';
		this->freeP += size + 1;
		this->freeSpace -= size + 1;
	} else{
		return NULL;
	}

	return tmp;
}

/*
===========================================================================

INFORMATION

===========================================================================
*/

Information::Information()
{
	lexem = NULL;
}

Information::~Information()
{
}

bool Information::compareLexem( char *lexem ) const
{
	return static_cast<bool>( !strcmp( this->lexem, lexem ) );
}

char *Information::getName() const
{
    return this->lexem;
}

TokenType Information::getType( void ) const
{
    return type;
}

void Information::setName( char *lexem )
{
	this->lexem = lexem;
}

void Information::setType( TokenType t )
{
    type = t;
}

/*
===========================================================================

SYMTABENTRY

===========================================================================
*/

SymtabEntry::SymtabEntry()
{
	next = NULL;
}

SymtabEntry::~SymtabEntry()
{
}

SymtabEntry *SymtabEntry::getNext( void )
{
	return next;
}

void SymtabEntry::setNext( SymtabEntry *ste )
{
	next = ste;
}

Information *SymtabEntry::getInfo()
{
    return &info;
}


/*
===========================================================================

SYMBOLTABLE

===========================================================================
*/

Symboltable::Symboltable()
{
	int i;

	for( i = 0; i < NUM_BUCKETS; i++ ) {
		bucket[i] = NULL;
	}

	strTab = new StringTab();

	initSymbols();
}

Symboltable::~Symboltable()
{
	int i;
	SymtabEntry *ste, *ste_temp;

    delete strTab;
	for( i = 0; i < NUM_BUCKETS; i++ ) {
		ste = bucket[i];
		while( ste ) {
			ste_temp = ste->getNext();
			delete ste;
			ste = ste_temp;
		}
	}

}

int Symboltable::hash( char *lexem )
{
	int len = strlen( lexem );
	if( len == 0 ) return 0;
	return static_cast<int>(*(lexem + len - 1)); // always a value [0, 255]
}

void Symboltable::initSymbols( void ) // Insert the keywords
{
	insert( (char*)"if" )->getInfo()->setType(T_IF);
	insert( (char*)"while" )->getInfo()->setType(T_WHILE);
	insert( (char*)"int" )->getInfo()->setType(T_INT);
	insert( (char*)"IF" )->getInfo()->setType(T_IF);
	insert( (char*)"WHILE" )->getInfo()->setType(T_WHILE);
}

symtabkey_t Symboltable::insert( char *lexem )
{
	char *strTabNode = NULL;
	int i = hash( lexem );
	SymtabEntry *ste = bucket[i];

	while( ste ) {
		if( ste->getInfo()->compareLexem( lexem ) ) {
			break;
		}

		ste = ste->getNext();
	}

	if( !ste ) { // If we get in, we insert the lexem that doesn't yet exists
		strTabNode = strTab->insert( lexem, strlen( lexem ) );
		if( !strTabNode ) {
            return NULL;
        }

		ste = new SymtabEntry();
		ste->getInfo()->setName( strTabNode );
		ste->getInfo()->setType(T_IDENTIFIER);
		ste->setNext( bucket[i] );
		bucket[i] = ste;
	}

	return ste;
}

/*
===========================================================================

PRINTING OF SYMTAB (REMOVE ME LATER)

===========================================================================
*/


void Symboltable::print( void )
{
	char *lexem;
	int i;
	SymtabEntry *ste;

	for( i = 0; i < NUM_BUCKETS; i++ ) {
		ste = bucket[i];
		while( ste ) {
			lexem = ste->getInfo()->getName();
			std::cout << "bucket[" << i << "]: "
                        "addrSte->" << static_cast<void*>(ste) << " "
                        "Lexem->" << lexem;
            std::cout << " Type: ";
            printTokenType(ste->getInfo()->getType());
            std::cout << std::endl;
			ste = ste->getNext();
		}
	}
}

